(*
Domtool (http://hcoop.sf.net/)
Copyright (C) 2004  Adam Chlipala

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*)

(* Utility functions *)

signature UTIL =
sig
    val impLoop : (unit -> 'a) -> ('a -> bool) -> ('a * 'b -> 'b) -> 'b -> unit
    (* (impLoop next stop body init) repeatedly executes next until stop returns
     * true on the result. Running state is kept in the style of foldl, with
     * the state for the next iteration being determiend by calling body with
     * the most recent next return and the current state.
     *)

    val ioLoop : (unit -> string) -> (string * 'b -> 'b) -> 'b -> unit
    (* impLoop specialized to work with TextIO.inputLine, where a return of ""
     * indicates end-of-file.
     *)
    val ioOptLoop : (unit -> string option) -> (string * 'b -> 'b) -> 'b -> unit
    (* Like ioLoop, but with NONE as sentinel value. *)

    val isIdent : char -> bool
    (* Is the character a valid part of an identifier? (lowercase letter or
     * digit?
     *)

    val validHost : string -> bool
    (* Valid Internet hostname with no periods? *)
    val validDomain : string -> bool
    (* Valid domain name? *)
    val validUser : string -> bool
    (* Valid UNIX username? *)
    val validEmail : string -> bool
    (* Valid e-mail address? *)
    val isTmp : string -> bool
    (* Is the name of a temporary file generated by an editor? *)
    val validIp : string -> bool
    (* Is a valid IPv4 address? *)

    val trimLast : string -> string
    (* Remove the last character from a string *)

    val toDir : string -> string
    (* Translates a domain name into a fileystem path *)

    val checkPath : StringSet.set * string -> bool
    (* Given a set of valid path bases and a path, is it allowable? *)
    val resolveAddr : string StringMap.map * string -> string
    (* Determine the value of an address field that may include variables.
     * Check that the final result is a validIp. *)
    val resolveDomain : string StringMap.map * string -> string
    (* Determine the value of a domain field that may include variables.
     * Check that the final result is a validDomain. *)
end
